/*
*******************************************
*   Developed by Luiz Fernando Romanini   *
*******************************************
*/

#include "Application.h"
#include "Exception.h"

Application *Application::hInstance = 0;

Application::Application( )
{
	Application::hInstance = this;
}

Application *Application::getApplication( )
{
	if( !Application::hInstance )
		throw Exception( __FILE__, __LINE__, "Cannot execute: Application::hInstance is null." );

	return Application::hInstance;
}

Application::~Application( )
{
}

void Application::setApplicationName( const char *szApplicationName )
{
	this->strApplicationName = szApplicationName;

	if( this->strApplicationName.find_last_of( '/' ) < this->strApplicationName.size( ) )
		this->strApplicationName = this->strApplicationName.substr( this->strApplicationName.find_last_of( '/' ) + 1 );
}

void Application::readArguments( int iArgCount, char *szArgValue[ ] )
{
	this->mapArgs.clear( );

	for( int i = 1; i < iArgCount; ++i )
	{
		std::string strArg = szArgValue[ i ];
		std::string::size_type pos = strArg.find_first_of( '=' );

		std::string strArgumentName = pos > strArg.size( ) ? strArg : strArg.substr( 0, pos );
		std::string strArgumentValue = pos > strArg.size( ) ? "" : strArg.substr( pos + 1 );

		this->mapArgs.insert(
			std::pair < std::string, std::string >( strArgumentName, strArgumentValue )
		);
	}
}

int Application::getArgumentsCount( ) const
{
	return this->mapArgs.size( );
}

bool Application::isArgumentSet( const std::string &strArgumentName ) const
{
	bool bIsArgSet = true;
	std::map < std::string, std::string >::const_iterator it = this->mapArgs.find( strArgumentName );

	if( it == this->mapArgs.end( ) )
		bIsArgSet = false;

	return bIsArgSet;
}

std::string Application::getArgumentValue( const std::string &strArgumentName ) const
{
	std::map < std::string, std::string >::const_iterator it = this->mapArgs.find( strArgumentName );

	if( it == this->mapArgs.end( ) )
		return "";

	return it->second;
}

const char * Application::getVersion( ) const
{
	return Application::VERSION;
}
