#include <errno.h>
#include <cstring>
#include <iostream>
#include <sstream>

#include "ServerSocket.h"
#include "SocketException.h"

ServerSocket::ServerSocket( )
{
	if( !this->Socket::create( ) )
	{
		std::ostringstream ossWhat;
		ossWhat << "Unable to create server socket : " << std::strerror( errno );

		throw SocketException( __FILE__, __LINE__, ossWhat.str( ) );
	}
}

ServerSocket::~ServerSocket( )
{
}

void ServerSocket::accept( int iPort, int iMaxConnections )
{
	if( !this->Socket::bind( iPort ) )
	{
		std::ostringstream ossWhat;

		ossWhat
			<< "Could not bind to port [ "
			<< iPort
			<< " ] : "
			<< std::strerror( errno )
		;

		throw SocketException( __FILE__, __LINE__, ossWhat.str( ) );
	}

	if( !this->Socket::listen( iMaxConnections ) )
	{
		std::ostringstream ossWhat;

		ossWhat
			<< "Could not listen [ "
			<< iMaxConnections
			<< " ] connection(s) : "
			<< std::strerror( errno )
		;

		throw SocketException( __FILE__, __LINE__, ossWhat.str( ) );
	}

	if( !this->Socket::accept( this ) )
	{
		std::ostringstream ossWhat;
		ossWhat << "Could not accept from socket : " << std::strerror( errno );

		throw SocketException( __FILE__, __LINE__, ossWhat.str( ) );
	}
}

void ServerSocket::send( const std::string &strMessage ) const
{
	if( !this->Socket::send( strMessage ) )
	{
		std::ostringstream ossWhat;
		ossWhat << "Could not write to socket : " << std::strerror( errno );

		throw SocketException( __FILE__, __LINE__, ossWhat.str( ) );
	}
}

std::string ServerSocket::receive( ) const
{
	std::string strMessage;

	switch( this->Socket::receive( &strMessage ) )
	{
	case -1:
		// Error in Socket::receive
		throw SocketException( __FILE__, __LINE__, std::string( "Could not read from socket : " ) + std::strerror( errno ) );
		break;

	case 0:
		// Client disconneted
		throw SocketException( __FILE__, __LINE__, std::string( "Client disconnected : " ) + std::strerror( errno ) );
		break;

	default:
		break;
	}

	return strMessage;
}
