#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <cstdlib>
#include <cstring>

#include "Socket.h"

const int Socket::MAX_RECV = 8192;

Socket::Socket( ) : iSock( -1 )
{
	memset( &this->addr, 0, sizeof this->addr );
}

Socket::~Socket( )
{
	this->disconnect( );
}

bool Socket::create( )
{
	int iOn = 1;

	this->iSock = socket( AF_INET, SOCK_STREAM, 0 );

	if( !this->isValid( ) )
		return false;

	if( setsockopt( this->iSock,
		SOL_SOCKET,
		SO_REUSEADDR,
		reinterpret_cast< const char * >( &iOn ),
		sizeof iOn ) == -1 )
	{
		return false;
	}

	return true;
}

bool Socket::bind( int iPort )
{
	if( !this->isValid( ) )
		return false;

	this->addr.sin_family = AF_INET;
	this->addr.sin_addr.s_addr = INADDR_ANY;
	this->addr.sin_port = htons( iPort );

	if( ::bind( this->iSock,
		reinterpret_cast< struct sockaddr * >( &this->addr ),
		sizeof this->addr ) == -1 )
	{
		return false;
	}

	return true;
}

bool Socket::listen( int iMaxConnections ) const
{
	if( !this->isValid( ) )
		return false;

	if( ::listen( this->iSock, iMaxConnections ) == -1 )
		return false;

	return true;
}

bool Socket::accept( Socket *s )
{
	int iAddrLength = sizeof this->addr;

	s->iSock = ::accept( this->iSock,
		reinterpret_cast< sockaddr * >( &this->addr ),
		reinterpret_cast< socklen_t * >( &iAddrLength ) );

	if( s->iSock <= 0 )
		return false;

	return true;
}

bool Socket::connect( const std::string &strHost, int iPort )
{
	if( !this->isValid( ) )
		return false;

	this->addr.sin_family = AF_INET;
	this->addr.sin_port = htons( iPort );

	inet_pton( AF_INET, strHost.c_str( ), &this->addr.sin_addr );

	if( errno == EAFNOSUPPORT )
		return false;

	if( ::connect( this->iSock,
		reinterpret_cast< sockaddr * >( &this->addr ),
		sizeof this->addr ) != 0 )
	{
		return false;
	}

	return true;
}

void Socket::disconnect( )
{
	if( this->isValid( ) )
	{
		close( this->iSock );
		this->iSock = -1;
	}
}

bool Socket::send( const std::string &strMessage ) const
{
	if( ::send( this->iSock, strMessage.c_str( ), strMessage.size( ), MSG_WAITALL ) == -1 )
		return false;
	else
		return true;
}

int Socket::receive( std::string *strMessage ) const
{
	char szBuffer[ Socket::MAX_RECV ] = { 0 };
	int iStatus = recv( this->iSock, szBuffer, Socket::MAX_RECV, 0 );

	*strMessage = "";

	switch( iStatus )
	{
	case -1:
		// Error in Socket::receive
		break;

	case 0:
		break;

	default:
		*strMessage = szBuffer;
		break;
	}

	return iStatus;
}

void Socket::setNonBlocking( bool b )
{
	int iOpts = fcntl( this->iSock, F_GETFL );

	if( iOpts < 0 )
		return;

	if( b )
		iOpts |= O_NONBLOCK;
	else
		iOpts &= ~O_NONBLOCK;

	fcntl( this->iSock, F_SETFL, iOpts );
}

bool Socket::isValid( ) const
{
	return this->iSock != -1;
}
