/*
*******************************************
*   Developed by Luiz Fernando Romanini   *
*******************************************
*/

#include <cstdlib>
#include <iostream>
#include <sstream>

#include "Exception.h"
#include "ServerSocket.h"

#include "VHappy.h"
#include "VREPRobot.h"

const char Application::VERSION[ ] = "v1.0.0";

VHappy theApplication;

VHappy::VHappy( )
{
	this->simxClientId = -1;
}

VHappy::~VHappy( )
{
	this->disconnect( );
}

int VHappy::main( )
{
	std::string strIpAddress = static_cast< std::string >( "127.0.0.1" );
	int iPort = 19997;
	int iVHappyPort = 35000;

	if( this->getArgumentValue( "--vrep_ip" ) != "" )
		strIpAddress = this->getArgumentValue( "--vrep_ip" );

	if( this->getArgumentValue( "--vrep_port" ) != "" )
		iPort = atoi( this->getArgumentValue( "--vrep_port" ).c_str( ) );

	if( this->getArgumentValue( "--vHappy_port" ) != "" )
		iVHappyPort = atoi( this->getArgumentValue( "--vHappy_port" ).c_str( ) );

	// Connect to v-Rep
	std::cout << "Trying to connect to v-rep on [ " << strIpAddress << " : " << iPort << " ]" << std::endl;
	this->connect( strIpAddress, iPort );
	std::cout << "Connected to v-rep!" << std::endl;

	VREPRobot vrepRobot( this->simxClientId );

	// Accept connections from Clarion
	ServerSocket serverSocket;

	std::cout << "Waiting for connections in port [ " << iVHappyPort << " ]..." << std::endl;
	serverSocket.accept( iVHappyPort );
	std::cout << "Client connected!" << std::endl;

	while( true )
	{
		std::string strReceived = serverSocket.receive( );
		std::string strReceivedTruncated = strReceived.substr( 0, strReceived.find_last_of( '\n' ) - 1 );

		std::cout << "Received command [ " << strReceivedTruncated << " ] " << std::endl;

		if( strReceivedTruncated == "moveLeft" )
		{
			vrepRobot.moveLeft( );
		}
		else if( strReceivedTruncated == "moveFront" )
		{
			vrepRobot.moveFront( );
		}
		else if( strReceivedTruncated == "stop" )
		{
			vrepRobot.stop( );
		}
		else if( strReceivedTruncated == "moveReverse" )
		{
			vrepRobot.moveReverse( );
		}
		else if( strReceivedTruncated == "moveRight" )
		{
			vrepRobot.moveRight( );
		}
		else if( strReceivedTruncated == "bye" )
		{
			std::cout << "Exiting..." << std::endl;
			break;
		}
		else
		{
			std::cout << "Nothing to do with command [ " << strReceivedTruncated << " ]" << std::endl;
		}
	}

	this->disconnect( );
	return 0;
}

void VHappy::connect( const std::string &strIpAddress, int iPort )
{
	const int WAIT_UNTIL_CONNECTED = 1;
	const int DO_NOT_RECONNECT_ONCE_DISCONNECTED = 0;
	const int TIMEOUT_IN_MS = 10000;
	const int COMM_THREAD_CYCLE_IN_MS = 5;

	this->simxClientId = simxStart(
		strIpAddress.c_str( ),
		iPort,
		WAIT_UNTIL_CONNECTED,
		DO_NOT_RECONNECT_ONCE_DISCONNECTED,
		TIMEOUT_IN_MS,
		COMM_THREAD_CYCLE_IN_MS
	);

	if( this->simxClientId == -1 )
		throw Exception( __FILE__, __LINE__, "Unable to connect to v-rep!" );

	simxStartSimulation( this->simxClientId, simx_opmode_oneshot_wait );
}

void VHappy::disconnect( )
{
	if( this->simxClientId != -1 )
	{
		simxStopSimulation( this->simxClientId, simx_opmode_oneshot_wait );
		simxFinish( this->simxClientId );
		this->simxClientId = -1;
	}
}
